// Easy New Notebook
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1360 $ on $Date: 2010-07-20 13:13:48 -0700 (Tue, 20 Jul 2010) $

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version = 2.1

#pragma ModuleName=EasyNewNotebook
#pragma hide=1

#include "ScreenSizer", optional

// TEST MODE (comment out for actual use)
//#define TESTME

// Procedure String Constants
// define the default file folder and its name

Static StrConstant thePackage="EasyNewNotebook"
Static StrConstant thePackageFolder="root:Packages:EasyNewNotebook"
Static StrConstant theProcedureFile = "EasyNewNotebook.ipf"
Static StrConstant thePackageAuthor="Jeffrey J Weimer"
Static StrConstant thePackageInfo="An easy way to initialize the setup of a notebook"
Static Constant thePackageVersion = 2.1
Static Constant hasHelp = 1
Static Constant removable = 1

Static Constant kENNWidth = 265
#if (cmpstr(IgorInfo(2),"Macintosh")==0)
Static Constant kENNHeight = 135
Static Constant kENNNarrowHeight = 40
#else
Static Constant kENNHeight = 105
Static Constant kENNNarrowHeight = 30
#endif

Static Constant kPrefByteCount = 83
Static StrConstant kPackageName = "EasyNewNotebook"
Static StrConstant kPreferenceFileName = "EasyNewNotebookPrefs.bin"

// Preferences A

Static Structure ENNPrefsA

	double ntbkCoords[4]	// left, top, right, bottom
	int16 fsize
	uchar kill
	uchar title[32]
	uchar font[16]

EndStructure

// Save Package Prefs

Static Function SavePackagePrefs(sw)
	variable sw
	
	NVAR/SDFR=root:Packages:EasyNewNotebook left, right, top, bottom, fsize, kill
	SVAR/SDFR=root:Packages:EasyNewNotebook font, title
	
	Struct ENNPrefsA pA
	
	pA.ntbkCoords[0] = left
	pA.ntbkCoords[1] = right
	pA.ntbkCoords[2] = top
	pA.ntbkCoords[3] = bottom
	
	pA.fsize = fsize
	pA.kill = kill
	pA.title = title
	pA.font = font
	
	SavePackagePreferences kPackageName, kPreferenceFileName, 0, pA
	
	return 0
	
end

// Load Package Prefs

Static Function LoadPackagePrefs(sw)
	variable sw
	
	NVAR/SDFR=root:Packages:EasyNewNotebook left, right, top, bottom, fsize, kill
	SVAR/SDFR=root:Packages:EasyNewNotebook font, title
	
	Struct ENNPrefsA pA
	
	LoadPackagePreferences/MIS=1 kPackageName, kPreferenceFileName, 0, pA
	
	if (V_flag!=0 || V_bytesRead==0)
		return -1
	endif
	
	// current byte count of preference file
	
	if (V_bytesRead!=kPrefByteCount)
		fsize = 12
		kill = 0
		font = GetDefaultFont("")
		title = "ENNotebook"
		SavePackagePrefs(0)
		return -1
	endif
	
	left = pA.ntbkCoords[0]
	right = pA.ntbkCoords[1]
	top = pA.ntbkCoords[2]
	bottom = pA.ntbkCoords[3]
	
	fsize = pA.fsize
	kill = pA.kill
	title = pA.title
	font = pA.font
	
	return 0
	
end

// Set Menu Hook

Static Function IgorMenuHook(isSel, menuStr, itemStr, itemNo, topWindowName, wt)
	Variable isSel, itemNo, wt
	String menuStr, itemStr, topWindowName

	Variable handled= 0
	string theCmd=""
	switch(isSel)
		case 0:
			break
		case 1:
			if ( (cmpstr(menuStr,"New") == 0) && (cmpstr(itemStr,"Notebook") == 0) )
				EasyNewNotebook#CreateNewNotebook(0)
				handled= 1
			endif
			break
	endswitch
	return handled
End

Function SetupEasyNewNotebook()
	
	SVAR/Z pt = root:Packages:PackageTools:Keys:EasyNewNotebook
	string theCmd

	if (!SVAR_Exists(pt))
		sprintf theCmd, "\"%s\"", thePackage
		sprintf theCmd, "%s,file=\"%s\"", theCmd, theProcedureFile
		sprintf theCmd, "%s,info=\"%s\"", theCmd, thePackageInfo
		sprintf theCmd, "%s,author=\"%s\"", theCmd, thePackageAuthor
		sprintf theCmd, "%s,version=%g", theCmd, thePackageVersion
		sprintf theCmd, "%s,hasHelp=%d", theCmd, hasHelp
		sprintf theCmd, "%s,removable=%d", theCmd, removable
		sprintf theCmd, "PackageSetup(%s)", theCmd
		Execute/P/Q theCmd
	else
		sprintf theCmd, "\"%s\"", thePackage
		sprintf theCmd, "%s,file=\"%s\"", theCmd, theProcedureFile
		sprintf theCmd, "%s,info=\"%s\"", theCmd, thePackageInfo
		sprintf theCmd, "%s,author=\"%s\"", theCmd, thePackageAuthor
		sprintf theCmd, "%s,version=%g", theCmd, thePackageVersion
		sprintf theCmd, "%s,hasHelp=%d", theCmd, hasHelp
		sprintf theCmd, "%s,removable=%d", theCmd, removable
		sprintf theCmd, "PackageUpdate(%s)", theCmd
		Execute/P/Q theCmd
	endif
	return 0
end

// Create New Notebook

// initialize the package as needed and put up the panel

Static Function CreateNewNotebook(how)
	variable how
	
	if (WinType("ENN_Panel")!=0)
		DoWindow/F ENN_Panel
		return 0
	endif
	DFREF cdf=GetDataFolderDFR()
	if (DataFolderExists(thePackageFolder)==0)
		SetDataFolder root:
		NewDataFolder/O/S Packages
		NewDataFolder/O/S EasyNewNotebook
		string/G title="ENNotebook"
		variable/G left, right, top, bottom
#if(exists("ScreenSizer#Initialize")==6)
			left = 5
			top = 5
			right=30
			bottom = 80
#else
			left = 10
			top = 10
			right = 200
			bottom = 300
#endif
	endif
	
	// include new parameters (as of version 1.0)
	SVAR/Z dfont = root:Packages:EasyNewNotebook:font
	if (!SVAR_exists(dfont))
		SetDataFolder $thePackageFolder
		string/G font = GetDefaultFont("")
		variable/G fsize = 12
		variable/G kill = 0
	endif
	SetDataFolder cdf
	SetUpEasyNewNotebook()
	
	ShowPanel(how)
	return 0
end

// Show Panel
// put up the panel
// 0: new panel
// >0: just draw the buttons with top offset = how!!!!!
// -1: remove buttons added above!!!
// -2: redisplay buttons removed above!!!

Static Function ShowPanel(how)
	variable how
	
	NVAR/SDFR=root:Packages:EasyNewNotebook left, right, top, bottom, fsize, kill
	SVAR/SDFR=root:Packages:EasyNewNotebook font, title
	
	variable prefs
	prefs=LoadPackagePrefs(0)

	// create the panel

	switch(how)
	
		// show certain panel controls
		
		case -2:
			SetVariable enn_left, disable=0
			SetVariable enn_top, disable=0
			SetVariable enn_bottom, disable=0
			SetVariable enn_right, disable=0
			PopupMenu enn_killstate, disable=0
			CheckBox enn_format, disable=0
			ControlInfo enn_format
			CheckBox enn_ruler, disable=!(V_value)
			PopupMenu enn_name, disable=0
			ControlInfo enn_WD
			if (V_flag!=0)
				PopupMenu enn_WD, disable=0
			endif
			title = "ENNotebook"
			break
	
		// remove certain panel controls
		
		case -1:
			SetVariable enn_left, disable=1
			SetVariable enn_top, disable=1
			SetVariable enn_bottom, disable=1
			SetVariable enn_right, disable=1
			PopupMenu enn_killstate, disable=1
			CheckBox enn_format, disable=1
			CheckBox enn_ruler, disable=1
			PopupMenu enn_name, disable=1
			ControlInfo enn_WD
			if (V_flag!=0)
				PopupMenu enn_WD, disable=1
			endif
			break
			
		// create the panel
		
		case 0:
			title = "ENNotebook"
			string pname = "ENN Panel"

// use ScreenSizer if it exists
// make sure sizes are within bounds
			
#if(exists("ScreenSizer#Initialize")==6)
				left = left*(left<100)
				top = top*(top<100)
				right = right*(right<100)
				right += 30*(right==0)
				bottom = bottom*(bottom<100)
				bottom += 50*(bottom==0)
				pname += " +SS"
				string theCmd =""
				sprintf theCmd "ScreenSizer#MakePanel(30,15,%d,%d,\"%s\",other=\"/K=1/N=ENN_Panel\")", kENNWidth, kENNHeight,pname
				Execute/Q/Z theCmd
#else
				NewPanel/K=1/N=ENN_Panel/W=(20,20,20+kENNWidth,20+kENNHeight) as pname
#endif

			killstrings/Z root:S_name
			
// in test mode, allow panel edits and do not use panel hook

#ifndef TESTME
			ModifyPanel noEdit=1,fixedSize=1
			SetWindow ENN_Panel,hook(ENN)=EasyNewNotebook#closeme
#else
			pname += " TEST"
#endif

// use WindowDesktops if it exists

			if(exists("ACL_WindowDesktops#Initialize_pnlDesktopControl")==6)
				pname += " +WD"
			endif

			// name the panel
	
			DoWindow/T ENN_Panel,pname

			// NOTEBOOK CONTROLS
			
			// title and name
			
			SetVariable enn_title,value=title,pos={7,7+how},size={160,16},title="Title", proc=EasyNewNotebook#GoOnChange

			// do it

			Button enn_doit,pos={178,4},size={40,20},proc=EasyNewNotebook#DoIt,title="Do It"
			
			// help button
			
			Button enn_gethelp,pos={225,4+how},size={15,20},proc=EasyNewNotebook#HelporRemove,title="?"
			
			// removeme button
			
			Button enn_removeme,pos={245,4+how},size={15,20},proc=EasyNewNotebook#HelporRemove,title="X"

			// open or close portion of panel
			
			Button enn_openorclose,pos={8,23},size={15,16},title="V",font="Arial",fSize=10
			Button enn_openorclose, proc=EasyNewNotebook#OpenorClose, userData="opened"
			SetDrawLayer UserBack
			DrawLine 25,32+how,262,32+how

		// put certain panel controls
		
		default:
		
			PopupMenu enn_name,pos={7,40+how},size={116,21},title="Name"
			PopupMenu enn_name,mode=1,popvalue="From Title",value= #"\"From Title;Time Stamp;\""
		
			// format or not
			
			CheckBox enn_format,pos={145,43+how},size={65,14},title="Format",value= 1,proc=EasyNewNotebook#SetFormat
			CheckBox enn_ruler,pos={210,43+how},size={43,14},title="Ruler",value= 0
		
			// kill state
			
			string kstr =StringFromList(kill,"Normal;Silent;NO KILL;Hide Only;")
			PopupMenu enn_killstate,pos={160,63+how},size={88,21},title="Kill",proc=EasyNewNotebook#SetKillState
			PopupMenu enn_killstate,mode=(kill+1),popvalue=kstr,value="Normal;Silent;NO KILL;Hide Only;"

// use WindowDesktops if it exists and a current desktop has been set

			if(exists("ACL_WindowDesktops#Initialize_pnlDesktopControl")==6)
				variable/G tmpcDD
				Execute/Q/Z "ACL_WindowDesktops#GetDisplayedDesktop(globalOutputVarPathStr=\"root:tmpcDD\")"
				NVAR cDD = root:tmpcDD
				if (numtype(cDD)==2)
					PopupMenu enn_WD title="",value="-;D;1;2;3;4;", pos={165,87+how}, mode=1
				else
					PopupMenu enn_WD title="",value="-;D;1;2;3;4;", pos={165,87+how}, mode=(cDD+2)
				endif
				killvariables root:tmpcDD
			endif

			// sizes
		
			SetVariable enn_left,pos={5,68+how},size={70,16},value=left
			SetVariable enn_top,pos={5,88+how},size={70,16},value=top
			SetVariable enn_right,pos={84,68+how},size={70,16},value=right
			SetVariable enn_bottom,pos={84,88+how},size={70,16},value=bottom

			// use screensizer if it exists
			
#if(exists("ScreenSizer#Initialize")==6)
				SetVariable enn_left, title="|<%", limits={5,100,5}
				SetVariable enn_top, title="^%", limits={5,100,5}
				SetVariable enn_bottom, title="v%", limits={5,100,5}
				SetVariable enn_right, title="|>%", limits={5,100,5}
#else
				SetVariable enn_left, title="|<",limits={10,600,10}
				SetVariable enn_top, title="^",limits={10,600,10}
				SetVariable enn_right, title="|>",limits={10,600,10}
				SetVariable enn_bottom, title="v",limits={10,600,10}
#endif
			
			// font settings
			
			variable wp = WhichListItem(num2str(fsize),"9;10;12;14;")+1
			PopupMenu enn_fontsize,title="Font ",pos={5,110+how},mode=wp,value="9;10;12;14;",popvalue=num2str(fsize),proc=EasyNewNotebook#SetFontSize
			PopupMenu enn_fontlist,title="pt   ",pos={75,110+how},mode=1,value=#"EasyNewNotebook#TheShortFontList()",proc=EasyNewNotebook#SetFont
			break
	endswitch
//	DoWindow/F ENN_Panel
	STRUCT WMButtonAction ba
	if (prefs<0)
		ba.UserData="closed"
	else
		ba.UserData="opened"
	endif
	ba.eventCode=-1
	OpenorClose(ba)
	DoUpdate
	SetVariable enn_title, activate
	return 0
End

Static Function SetPosition(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	
	NVAR/SDFR=root:Packages:EasyNewNotebook left, right, top, bottom

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			strswitch(sva.ctrlName)
				case "enn_top":
					top = sva.dval
					break
				case "enn_left":
					left = sva.dval
					break
				case "enn_right":
					right = sva.dval
					break
				case "enn_bottom":
					bottom = sva.dval
					break
			endswitch
			break
	endswitch

	return 0
End

Static Function SetKillState(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	NVAR kill = root:Packages:EasyNewNotebook:kill

	switch( pa.eventCode )
		case 2: // mouse up
			kill = pa.popNum-1
			break
	endswitch

	return 0
End

Static Function SetFontSize(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	NVAR fsize = root:Packages:EasyNewNotebook:fsize

	switch( pa.eventCode )
		case 2: // mouse up
			fsize = str2num(pa.popStr)
			break
	endswitch

	return 0
End

Static Function SetFont(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR dfont = root:Packages:EasyNewNotebook:font

	switch( pa.eventCode )
		case 2: // mouse up
			if (pa.popNum==1)
				dfont = GetDefaultFont("")
			else
				dfont = pa.popStr
			endif
			break
	endswitch

	return 0
End

Static Function/S TheShortFontList()

	SVAR dfont = root:Packages:EasyNewNotebook:font
	
	string flist=FontList(";")
	string rlist = "Default (" + dfont + ");"
	rlist += ListMatch(flist,"Arial")
	rlist += ListMatch(flist,"Helvetica")
	rlist += ListMatch(flist,"Geneva")
	rlist += ListMatch(flist,"Times")
	rlist += ListMatch(flist,"Times New Roman")
	rlist = RemoveFromList(dfont,rlist)
	
	return rlist
End

// Close Me
// close panel

Static Function CloseMe(ws)
	STRUCT WMWinHookStruct &ws

	SVAR title=root:Packages:EasyNewNotebook:title
	STRUCT WMButtonAction ba
	
	switch(ws.eventCode)
		case 1:
			if (cmpstr(title,"SetMe!")==0)
				title=""
				KillWindow ENN_Panel
				break
			endif
			beep
			title="SetMe!"
			DoWindow/F ENN_Panel
			SetVariable enn_title, activate
			beep
			break
		case 11:
			if (ws.keyCode==13)
				ba.win=ws.winname
				ba.eventCode=2
				ba.ctrlName="DoIt"
				ba.userData=""
				DoIt(ba)
			endif
			break
		default:
			break
	endswitch
	return 0
end

// Get Help

Static Function HelporRemove(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	string theCmd

	switch( ba.eventCode )
		case 2:
			strswitch(ba.ctrlName)
				case "enn_GetHelp":
					theCmd = "PackageHelp(\"EasyNewNotebook\")"
					Execute/P/Q/Z theCmd
					break
				case "enn_RemoveMe":
					DoWindow/K ENN_Panel
					theCmd = "PackageRemove(\"EasyNewNotebook\")"
					Execute/P/Q/Z theCmd
					break
			endswitch
			break
	endswitch
	
	return 0
end

// DoIt
// create new notebook

Static Function DoIt(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=root:Packages:EasyNewNotebook left, right, top, bottom, fsize, kill
	SVAR/SDFR=root:Packages:EasyNewNotebook font, title

	variable format, ruler, wd = -1
	string name=""

	switch( ba.eventCode )
		case 2:
			ControlInfo enn_format
			format = V_value
			ControlInfo enn_killstate
			kill = V_value-1
			ControlInfo enn_ruler
			ruler = V_value
			if (strlen(title)==0)
				if (strlen(ba.userData)==0)
					title="ENNotebook"
				else
					title=ba.userData
				endif
			endif
			ControlInfo enn_name
			if (V_value==1)
				name=CleanUpName(title,0)
			else
				name="ENNotebook"+CleanUpName(time(),0)
			endif
			sprintf name, "%s%d", name, ItemsInList(WinList((name+"*"),";","WIN:16"))
			ControlInfo enn_WD
			if (V_flag!=0)
				wd = V_value-2
			endif
			if (WinType("ENN_Panel"))
				KillWindow ENN_Panel
			endif
			if (strlen(title)==0)
				NewNotebook/K=(kill)/F=(format)
			else
				NewNotebook/K=(kill)/F=(format)/N=$name as title
			endif
			if (format)
				Notebook $name showRuler=(ruler)			
				Notebook $name ruler=Normal, rulerDefaults={font,fsize,0,(0,0,0)}
			endif
			
			string theCmd = ""
			
// use ScreenSizer if it exists

#if(exists("ScreenSizer#Initialize")==6)
				sprintf theCmd "ScreenSizer#MoveFrontWindow(%d,%d,%d,%d)", left, top, left+right, top+bottom
				Execute/Q/Z theCmd
#else
				MoveWindow left, top, left+right, top+bottom
#endif

// use Window Desktops if it exists

			if(exists("ACL_WindowDesktops#Initialize_pnlDesktopControl")==6)
				if (wd>=0)
					variable/G tmpcDD
					Execute/Q/Z "ACL_WindowDesktops#GetDisplayedDesktop(globalOutputVarPathStr=\"root:tmpcDD\")"
					NVAR cDD = root:tmpcDD
					if (numtype(cDD)==2)
						sprintf theCmd "ACL_WindowDesktops#Initialize_pnlDesktopControl()"
						Execute/Q/Z theCmd
					endif
					killvariables root:tmpcDD
					sprintf theCmd "ACL_WindowDesktops#SetWindowDesktop(\"%s\",%d)", name, wd
					Execute/Q/Z theCmd
				endif
			endif
			SavePackagePrefs(0)
			break
	endswitch
	ba.userData=name

	return 0
End

// Go On Change makes a return automatically create a new notebook if the text in the title changes

Function GoOnChange(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	STRUCT WMButtonAction ba

	switch( sva.eventCode )
		case 2: // Enter key
//		case 3: // Live update
			if (stringmatch(sva.sval,"ENNotebook")==0)
				ba.eventCode=2
				DoIt(ba)
//				EasyNewNotebook#DoIt(ba)
			else
				SetVariable enn_title, activate
			endif
			break
	endswitch

	return 0
End

// Open or Close expands or collapses the panel size

Static  Function OpenorClose(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case -1: // forced
		case 2: // mouse up
			GetWindow kwTopWin wsize
			strswitch(ba.userData)
				case "opened":
					MoveWindow V_left, V_top, V_right, V_top+kENNNarrowHeight
					Button enn_OpenorClose title=">", userData="closed"
					break
				case "closed":
					MoveWindow V_left, V_top, V_right,V_top+kENNHeight
					Button enn_OpenorClose title="v",userData="opened"
					break
			endswitch
			break
	endswitch

	return 0
End

// SetFormat
// toggle on/off the ruler CheckBox enn_depending on format checkbox

Static Function SetFormat(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	switch( cba.eventCode )
		case 2: // mouse up
			CheckBox enn_ruler, disable=!cba.checked
			break
	endswitch

	return 0
End
